@@ -3,7 +3,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis-python
 #
-# Most of this work is copyright (C) 2013-2015 David R. MacIver
+# Most of this work is copyright (C) 2013-2018 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -30,7 +30,8 @@ from __future__ import division, print_function, absolute_import
 import inspect
 import traceback
 from unittest import TestCase
-from collections import namedtuple
+
+import attr
 
 import hypothesis.internal.conjecture.utils as cu
 from hypothesis.core import find
@@ -105,7 +106,7 @@ def run_state_machine_as_test(state_machine_factory, settings=None):
     except NoSuchExample:
         return
     try:
-        with BuildContext(is_final=True):
+        with BuildContext(None, is_final=True):
             breaker.run(state_machine_factory(), print_steps=True)
     except StopTest:
         pass
@@ -122,11 +123,16 @@ class GenericStateMachine(object):
     The intent is for it to be subclassed to provide state machine descriptions
 
     The way this is used is that Hypothesis will repeatedly execute something
-    that looks something like:
+    that looks something like::
 
-    x = MyStatemachineSubclass()
-    for _ in range(n_steps):
-        x.execute_step(x.steps().example())
+        x = MyStatemachineSubclass()
+        x.check_invariants()
+        try:
+            for _ in range(n_steps):
+                x.execute_step(x.steps().example())
+                x.check_invariants()
+        finally:
+            x.teardown()
 
     And if this ever produces an error it will shrink it down to a small
     sequence of example choices demonstrating that.
@@ -160,6 +166,10 @@ class GenericStateMachine(object):
         """
         pass
 
+    def check_invariants(self):
+        """Called after initializing and after executing each step."""
+        pass
+
     _test_case_cache = {}
 
     TestCase = TestCaseProperty()
@@ -176,9 +186,13 @@ class GenericStateMachine(object):
                 min_satisfying_examples=1
             )
 
-            def runTest(self):
-                run_state_machine_as_test(state_machine_class)
+        # We define this outside of the class and assign it because you can't
+        # assign attributes to instance method values in Python 2
+        def runTest(self):
+            run_state_machine_as_test(state_machine_class)
 
+        runTest.is_hypothesis_test = True
+        StateMachineTestCase.runTest = runTest
         base_name = state_machine_class.__name__
         StateMachineTestCase.__name__ = str(
             base_name + u'.TestCase'
@@ -192,6 +206,7 @@ class GenericStateMachine(object):
         )
         return StateMachineTestCase
 
+
 GenericStateMachine.find_breaking_runner = classmethod(find_breaking_runner)
 
 
@@ -215,22 +230,24 @@ class StateMachineRunner(object):
 
         stopping_value = 1 - 1.0 / (1 + self.n_steps * 0.5)
         try:
+            state_machine.check_invariants()
+
             steps = 0
             while True:
-                if steps == self.n_steps:
+                if steps >= self.n_steps:
                     stopping_value = 0
                 self.data.start_example()
                 if not cu.biased_coin(self.data, stopping_value):
                     self.data.stop_example()
                     break
-
+                assert steps < self.n_steps
                 value = self.data.draw(state_machine.steps())
                 steps += 1
-                if steps <= self.n_steps:
-                    if print_steps:
-                        state_machine.print_step(value)
-                    state_machine.execute_step(value)
+                if print_steps:
+                    state_machine.print_step(value)
+                state_machine.execute_step(value)
                 self.data.stop_example()
+                state_machine.check_invariants()
         finally:
             state_machine.teardown()
 
@@ -244,11 +261,13 @@ class StateMachineSearchStrategy(SearchStrategy):
         return StateMachineRunner(data, self.program_size)
 
 
-Rule = namedtuple(
-    u'Rule',
-    (u'targets', u'function', u'arguments', u'precondition',
-     u'parent_rule')
-)
+@attr.s()
+class Rule(object):
+    targets = attr.ib()
+    function = attr.ib()
+    arguments = attr.ib()
+    precondition = attr.ib()
+
 
 self_strategy = runner()
 
@@ -267,8 +286,10 @@ class Bundle(SearchStrategy):
         bundle.insert(integer_range(data, 0, len(bundle)), reference)
         return machine.names_to_values[reference.name]
 
+
 RULE_MARKER = u'hypothesis_stateful_rule'
 PRECONDITION_MARKER = u'hypothesis_stateful_precondition'
+INVARIANT_MARKER = u'hypothesis_stateful_invariant'
 
 
 def rule(targets=(), target=None, **kwargs):
@@ -294,16 +315,15 @@ def rule(targets=(), target=None, **kwargs):
         converted_targets.append(t)
 
     def accept(f):
-        parent_rule = getattr(f, RULE_MARKER, None)
-        if parent_rule is not None:
+        existing_rule = getattr(f, RULE_MARKER, None)
+        if existing_rule is not None:
             raise InvalidDefinition(
                 'A function cannot be used for two distinct rules. ',
                 Settings.default,
             )
         precondition = getattr(f, PRECONDITION_MARKER, None)
         rule = Rule(targets=tuple(converted_targets), arguments=kwargs,
-                    function=f, precondition=precondition,
-                    parent_rule=parent_rule)
+                    function=f, precondition=precondition)
 
         @proxies(f)
         def rule_wrapper(*args, **kwargs):
@@ -314,7 +334,9 @@ def rule(targets=(), target=None, **kwargs):
     return accept
 
 
-VarReference = namedtuple(u'VarReference', (u'name',))
+@attr.s()
+class VarReference(object):
+    name = attr.ib()
 
 
 def precondition(precond):
@@ -348,14 +370,63 @@ def precondition(precond):
             setattr(precondition_wrapper, PRECONDITION_MARKER, precond)
         else:
             new_rule = Rule(targets=rule.targets, arguments=rule.arguments,
-                            function=rule.function, precondition=precond,
-                            parent_rule=rule.parent_rule)
+                            function=rule.function, precondition=precond)
             setattr(precondition_wrapper, RULE_MARKER, new_rule)
+
+        invariant = getattr(f, INVARIANT_MARKER, None)
+        if invariant is not None:
+            new_invariant = Invariant(function=invariant.function,
+                                      precondition=precond)
+            setattr(precondition_wrapper, INVARIANT_MARKER, new_invariant)
+
         return precondition_wrapper
     return decorator
 
 
-ShuffleBundle = namedtuple('ShuffleBundle', ('bundle', 'swaps'))
+@attr.s()
+class Invariant(object):
+    function = attr.ib()
+    precondition = attr.ib()
+
+
+def invariant():
+    """Decorator to apply an invariant for rules in a RuleBasedStateMachine.
+    The decorated function will be run after every rule and can raise an
+    exception to indicate failed invariants.
+
+    For example::
+
+        class MyTestMachine(RuleBasedStateMachine):
+            state = 1
+
+            @invariant()
+            def is_nonzero(self):
+                assert self.state != 0
+
+    """
+    def accept(f):
+        existing_invariant = getattr(f, INVARIANT_MARKER, None)
+        if existing_invariant is not None:
+            raise InvalidDefinition(
+                'A function cannot be used for two distinct invariants.',
+                Settings.default,
+            )
+        precondition = getattr(f, PRECONDITION_MARKER, None)
+        rule = Invariant(function=f, precondition=precondition)
+
+        @proxies(f)
+        def invariant_wrapper(*args, **kwargs):
+            return f(*args, **kwargs)
+
+        setattr(invariant_wrapper, INVARIANT_MARKER, rule)
+        return invariant_wrapper
+    return accept
+
+
+@attr.s()
+class ShuffleBundle(object):
+    bundle = attr.ib()
+    swaps = attr.ib()
 
 
 class RuleBasedStateMachine(GenericStateMachine):
@@ -371,6 +442,7 @@ class RuleBasedStateMachine(GenericStateMachine):
 
     """
     _rules_per_class = {}
+    _invariants_per_class = {}
     _base_rules_per_class = {}
 
     def __init__(self):
@@ -420,18 +492,30 @@ class RuleBasedStateMachine(GenericStateMachine):
 
         for k, v in inspect.getmembers(cls):
             r = getattr(v, RULE_MARKER, None)
-            while r is not None:
+            if r is not None:
                 cls.define_rule(
                     r.targets, r.function, r.arguments, r.precondition,
-                    r.parent_rule
                 )
-                r = r.parent_rule
         cls._rules_per_class[cls] = cls._base_rules_per_class.pop(cls, [])
         return cls._rules_per_class[cls]
 
     @classmethod
-    def define_rule(cls, targets, function, arguments, precondition=None,
-                    parent_rule=None):
+    def invariants(cls):
+        try:
+            return cls._invariants_per_class[cls]
+        except KeyError:
+            pass
+
+        target = []
+        for k, v in inspect.getmembers(cls):
+            i = getattr(v, INVARIANT_MARKER, None)
+            if i is not None:
+                target.append(i)
+        cls._invariants_per_class[cls] = target
+        return cls._invariants_per_class[cls]
+
+    @classmethod
+    def define_rule(cls, targets, function, arguments, precondition=None):
         converted_arguments = {}
         for k, v in arguments.items():
             converted_arguments[k] = v
@@ -443,7 +527,6 @@ class RuleBasedStateMachine(GenericStateMachine):
         return target.append(
             Rule(
                 targets, function, converted_arguments, precondition,
-                parent_rule
             )
         )
 
@@ -452,7 +535,7 @@ class RuleBasedStateMachine(GenericStateMachine):
         for rule in self.rules():
             converted_arguments = {}
             valid = True
-            if rule.precondition is not None and not rule.precondition(self):
+            if rule.precondition and not rule.precondition(self):
                 continue
             for k, v in sorted(rule.arguments.items()):
                 if isinstance(v, Bundle):
@@ -512,3 +595,9 @@ class RuleBasedStateMachine(GenericStateMachine):
             )
             for target in rule.targets:
                 self.bundle(target).append(VarReference(name))
+
+    def check_invariants(self):
+        for invar in self.invariants():
+            if invar.precondition and not invar.precondition(self):
+                continue
+            invar.function(self)
